/*-----------------------------------------------------------------------------

 2002, Jeff M Soft und Hardware GmbH, All Rights Reserved

-----------------------------------------------------------------------------*/

#include "Module.h"
#include "SEMod_struct.h"
#include "SEPin.h"
#include <assert.h>


#include <fcntl.h>     /* for _O_TEXT and _O_BINARY */
#include <errno.h>     /* for EINVAL */
#include <sys\stat.h>  /* for _S_IWRITE */
#include <share.h>     /* for _SH_DENYNO */

//#include <sys/types.h>
#include <stdio.h> // files
#include <io.h>

#include <stdlib.h>
#include <string.h>
#include <process.h>
#include <wchar.h>




void Module::sub_process(long buffer_offset, long sampleFrames )
{ 
	float *rec   = buffer_offset + input1_buffer;
	if (*rec > 0.2)
	{
		REC = true;
	} 
	else
	{
		REC = false;
	}

	sampleFrames = 0;
}

// The core of a MIDI plugin
void Module::OnMidiData( unsigned long p_clock, unsigned long midi_msg, short pin_id)
{
 if(err==0)	
 {
	// system messages are for timing info etc
	// they don't have a channel, just pass them right through
	bool is_system_msg = ( midi_msg & SYSTEM_MSG ) == SYSTEM_MSG;

	if( !is_system_msg )
	{
		/*
		// The first byte of a midi message contains the status (note on/off etc) and the channel
		int chan = midi_msg & 0x0f;//channel
		int stat = midi_msg & 0xf0;//on off
		// the next two bytes depend on the type of message
		int b2 = ( midi_msg / 0x100 ) & 0xff; //
		int b3 = ( midi_msg / 0x10000 ) & 0xff;
        */

		
		if(REC == true)
		{ 
		  if((midi_msg & 0xf0) == NOTE_ON)
		  {
			r_midi = midi_msg;
            b2 = ( midi_msg / 0x100 ) & 0xff; //note
		    // b3 = ( midi_msg / 0x10000 ) & 0xff; //velo
			isit = b2;
              fwrite(&isit, sizeof(char),1, smidifileout);
		  }
		}
	}
	
 }
}

Module::Module(seaudioMasterCallback2 seaudioMaster, void *p_resvd1) : SEModule_base(seaudioMaster, p_resvd1)
{
}

void Module::open()
{
err = -1;
smidifileout  = NULL;
textfile = NULL;
isnew = false;
err2 = -1;

	SEModule_base::open();
	SET_PROCESS_FUNC(Module::sub_process);// choose which function is used to process audio
}

Module::~Module()
{
	if(err == 0)
	{
	  fclose(smidifileout);
	}

    if(err2 == 0)
	{
	  fclose(textfile);
	}
}

// describe the module
bool Module::getModuleProperties (SEModuleProperties* properties)
{
   LL dish;
   BB bish;
   sprintf_s(dish.thisSEM   ,100, "%s\n", "SNOTE_REC_1M" );
   int j;

   j  = sprintf_s(  bish.buffer    ,1000, "%s\n", " ************************************" );
   j  = sprintf_s(  bish.buffer + j,1000, "%s\n", " * 2009 December in Tbingen GERMANY " );
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " *            FAKE MUSIC             " ); 
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " *         by ralph.a.cronin         " ); 
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " ************************************" );
   j += sprintf_s(  bish.buffer + j,1000, "%s"  ,   dish.thisSEM );
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " *    RECORDER for SMIDI Files       " );
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " ************************************" );
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " *          enjoy SYNTHEDIT          " );
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " *        by Jeff Mc Clintock        " );
   j += sprintf_s(  bish.buffer + j,1000, "%s\n", " ************************************" );

   // MAXIS :
   // j += sprintf( buffer + j, "%s\n", " **************************************************************************************" );//120
   // j += sprintf( buffer + j, "%s\n", "XXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXXX" ); // = 54


	properties->name = dish.thisSEM;	// this is the name the end-user will see

	// return a uni+																																																			que string.
	// if posible include manufacturer and plugin identity
	// this is used internally by SE to identify the plug. No two plugs may have the same id.
	properties->id = dish.thisSEM;

	// Info, may include Author, Web page whatever
	properties->about =  bish.buffer;//"by ralph a. cronin and Jeff M";
	return true;


}

// describe the pins (plugs)
bool Module::getPinProperties (long index, SEPinProperties* properties)
{
	switch( index )
	{
	case 0:
		properties->name				= "MIDI In";
		properties->direction			= DR_IN;
		properties->datatype			= DT_MIDI2;
		break;
	case 1:	
		properties->name				= "REC";
		properties->variable_address	= &input1_buffer;
		properties->direction			= DR_IN;
		properties->datatype			= DT_FSAMPLE;
		properties->default_value		= "0";
        properties->datatype_extra      = "10,0,10,0";
		break;
	case 2:
		properties->name                = "OUTFILE";
        properties->variable_address    = &text_input;
        properties->direction           = DR_IN;
        properties->datatype            = DT_TEXT; 
        properties->flags               = IO_FILENAME;
        properties->datatype_extra      ="snote"; // file extension
        break;
	case 3:
		properties->name				= "on D: DIAGNOSE.TXT";
		properties->variable_address	= &admin;
		properties->direction			= DR_PARAMETER;
        properties->datatype			= DT_BOOL;
		properties->default_value		= "false";
		break;
	default:
		return false; // host will ask for plugs 0,1,2,3 etc. return false to signal when done
	};

	return true;
}

void Module::diag(char * melde)
{
 err2= fopen_s(&textfile,"D:\\DIAGNOSE.TXT","a+");
 if(err2 == 0)
 {
	 fwrite(melde, sizeof(char),strlen(melde), textfile);
     fclose(textfile);
 }
}

void Module::OnPlugStateChange(SEPin *pin)
{ 
	in_stat = getPin(PN_TEXT)->getStatus(); // bei 1 wechsel
    if(in_stat == ((state_type)1))
	{
	 CallHost(seaudioMasterResolveFilename,PN_TEXT, MAX_FILENAME_LENGTH, (void *) full_path );
	 if(admin)
	 {
     sprintf_s(text,400,"%s %s\n","SNOTE_REC_1M",full_path);diag(text);
	 }
	 wcscpy_s((wchar_t *)&smidi_filename,strlen(full_path),(wchar_t *)&full_path);// copy name

     if(err == 0) fclose(smidifileout);
     err =  fopen_s(&smidifileout, smidi_filename,"a+b");
	 if(err == 0)
	 {
		 isnew =true;
		 waspause = false;
	 }
	}// here we got new path if !
}